% COMPARISON BETWEEN DIRECT AND ITERATIVE SOLVERS FOR LINEAR SYSTEMS
%
%    FD DISCRETIZATION OF
%
%            -Laplacian(u) + b . grad(u) = f,
%               u=0 on boundary
%
%       for b=[1 2 3], on the unit CUBE

% N: number of subdivisions in each coordinate direction
N=40;

% the one-dimnesional diference matrices
   e=ones(N-1,1);
   D2=(N^2)*spdiags([-e 2*e -e],[-1:1],N-1,N-1);
   D1=(N/2)*spdiags([-e e],[-1 1],N-1,N-1);
% building the true solution:
% First as an 3-d tensor
   h=1/N; x=[h:h:1-h]'; X=x*ones(1,N-1); Y=X';
   U=(4*X.*(1-X)).*(4*Y.*(1-Y)).*exp((X-0.2*Y).^2);
   U=reshape(U,numel(U),1);
   U=(U*ones(size(x'))).*(ones(size(U))*(x.*(1-x).^2)');
% and finally reshaped as a column vector
   ue=reshape(U,numel(U),1);

% building the 2-D FD matrix
   I=speye(N-1); %the identity matrix
   A1=(kron(D2,I)+kron(I,D2)); % the 2-d -Laplacian
   A2=kron(D1,I)+2*kron(I,D1); % the 2-d convection
   A=kron(I,A1)+kron(D2,speye(size(A1))); %the 3-d Laplacian
   A=A+kron(I,A2)+3*kron(D1,speye(size(A2))); % 3-d convection
   

% the rhs of the linear system to be solved
   b=A*ue;

% Solving the system by the grmes method and measuring
% elapsed time: Choose between the following
% two lines ...
   tic;[L,U]=ilu(A); tilu=toc, bb=U\(L\b);
   tic; [u,iflag,relres,iter,reshist]=gmres(A,b,N,1e-4,1,L,U); tg=toc
% for preconditioned method and the next one for method withoud precond.
%bb=b; tic; [u,iflag,relres,iter,reshist]=gmres(A,b,N,1e-4,1); tg=toc

% showing convergence flag and number of iterations and 
iflag,iter
% plotting history reseidual an showing error by the gmres method
  figure(1);clf; semilogy(reshist/norm(bb))
  elerrorg=norm(u-ue)/norm(ue)

% Solving by a direct method for sparse (band) matrices and showing
% elapsed time
tic; x=A\b; td=toc