% SIMPLE PROGRAM FOR PARAMETER CONTINUATION.

% PROBLEM DATA
% FUNCTIONS:
fun=@funp; jacob=@jfunp;

% Set initial u and parameters or load them from a file
lambda=0; mu=sqrt(2); u=[0 1]';

% Datos de la curva a calcular
nmax=20; % number of points to compute parameter increment
h=0.01;  % parameter increment

% Data for  Newton's method
TOL=1e-10; maxiter=6; % Tolerance max. no. of iteration.

%%%%%%%%%  Continuation program stars here %%%%%%%%%%%%%%

% Output arrays to store results
% Lambdas and Ues for the parameter and u, nUes for norm(u)
Lambdas=zeros(1,nmax+1); Ues=zeros(length(u),nmax+1);
nUes=zeros(1,nmax+1);

% Filling in output arrays
n=0;
Lambdas(n+1)=lambda; Ues(:,n+1)=u; nUes(n+1)=norm(u);

% Main cicle here
disp('Computing approximations. Please wait ...')
disp(' ')
for n=1:nmax
    % Initial guess for Newton's method. If n>1 and extrapolation,
    % otherwise initial data
    if n>1
        u0=u+h*((u-Ues(:,n-1))/(lambda-Lambdas(n-1)));
    else
        u0=u;
    end
    lambdanew=lambda+h;
    [unew,iflag,iter]=newton(fun,jacob,u0,[lambdanew,mu],...
        TOL,TOL,maxiter);
    if abs(iflag)>0
        disp('CRANKY: NEWTON METHOD FAILED')
        % Adjusting output arrays before leaving
        Lambdas=Lambdas(1:n); Ues=Ues(:,1:n); nUes=nUes(1:n);
        break % Command break take us out of the main cicle.
    end
    % If we are here it means Newton's method converged
    % Udpdating lambda and u
    lambda=lambdanew; u=unew;
    Lambdas(n+1)=lambda; Ues(:,n+1)=u; nUes(n+1)=norm(u);
    % Plotting results to monitor progress
    figure(2); clf; plot(Lambdas(1:n+1),nUes(1:n+1),'.','MarkerSize',8)
    % clf borra el gr'afico que haya.
end
disp(' ... done.')
disp('Do not forget to store your results somewhere.')