% PROGRAM FOR KELLER'S PSEUDOARCLENGTH CONTINUATION
% (WITH ADATPTIVE STEPLENGTH AND LIMITS)

%%%%%%%%%%%%    PROBLEM DATA  (CHANGE AT WILL)   %%%%%%%%%%%%%%%%%%%
% FUNCTIONS:
fun=@funp; jacob=@jfunp;


% Data for the branch to be computed
nmax=120; % number of points to compute
h=0.01;  % parameter increment
hmax=0.1; hmin=1e-7; % maximum and minimun increment allowed
lambdamax=0.5; lambdamin=-0.5; % maximum and minimun parameter
% allowed

% INITIAL DATA
% Set initial u, parameters and tangent
% Either manually  as in these three lines ...
   lambda=0; mu=sqrt(2); u=[0 1]';
   % we set a fictious tanget first step is parameter continuation
   t=[zeros(size(u));1];
   % All parameters to array params. Index of paraeter to vary in jpar
   params=[lambda,mu]; jpar=1;
% or read u t params and jpar from a file; change the name of the file
%   load init_data_7pi16; lambda=params(jpar); clear mu

% Data for  Newton's method
TOL=1e-10; maxiter=6; % Tolerance max. no. of iteration.

% FontSize for plots
FntS=18;


%%%%%%%%%%%%    END OF PROBLEM DATA  (STOP CHANGING)   %%%%%%%%%%%%%%%


%%%%%%%%%  Continuation program stars here %%%%%%%%%%%%%%

% Output arrays to store results
% Lambdas and Ues for the parameter and u, nUes for norm(u)
% Tes for the tangents, Iters for Newton iterations
% Hs for incrmentes
Lambdas=zeros(1,nmax+1); Ues=zeros(length(u),nmax+1);
nUes=zeros(1,nmax+1); Tes=zeros(length(t),nmax+1);
Iters=zeros(1,nmax+1); Hs=zeros(1,nmax+1);

% Filling in output arrays
n=0;
Lambdas(n+1)=lambda; Ues(:,n+1)=u; nUes(n+1)=norm(u);
Tes(:,n+1)=t; Hs(n+1)=h;

% Main cicle here
disp('Computing approximations. Please wait ...')
disp(' ')

m=length(u);
for n=1:nmax
    % Cicle to compute new approximation:
    % If Newton fails, h is halved
    % cheking limmits
    if lambda+t(m+1)*h>lambdamax;
        t=[zeros(size(u));1]; h=lambdamax-lambda;
    end
    if lambda+t(m+1)*h<lambdamin;
        t=[zeros(size(u));-1]; h=lambda-lambdamin;
    end
    iflag=1; h=2*h;
    while abs(iflag)>0 && h>hmin*max(1,norm([u; lambda]))
        h=h/2;
        % Initial guess for Newton's method: tangent extrapolation
        u0=u+h*t(1:m); lambda0=lambda+h*t(m+1);
        % reference point: current point
        ur=[u; lambda]; parmasc(jpar)=lambda0;
        [unew,lambdanew,iflag,iter,tnew]=newton_ps(fun,jacob,u0,...
            params,1,t,ur,h,TOL,TOL,maxiter);
    end
    if abs(iflag)>0
        disp('OH,CRANKY! NEWTON''S METHOD FAILED')
        % Adjusting output arrays before leaving
        Lambdas=Lambdas(1:n); Ues=Ues(:,1:n); nUes=nUes(1:n);
        Tes=Tes(:,1:n); Iters=Iters(1:n);
        break % Command break take us out of the main cicle.
    end
    % If we are here it means Newton's method converged
    % Udpdating lambda and u
    lambda=lambdanew; u=unew; t=tnew;
    Lambdas(n+1)=lambda; Ues(:,n+1)=u; nUes(n+1)=norm(u);
    Tes(:,n+1)=t; Iters(n+1)=iter; Hs(n+1)=h;
    % adjusting incrment h according to iterarions in Newton's method
    if iter<3; h=min(2*h,hmax*max(1,norm([u; lambda]))); end
    if iter>3; h=max(h/2,hmin*max(1,norm([u; lambda]))); end
    % Plotting results to monitor progress
    figure(2); clf; plot(Lambdas(1:n+1),nUes(1:n+1),'.','MarkerSize',8)
    set(gca,'FontSize',14); xlabel('\lambda','FontSize',FntS)
    ylabel('||u}||','FontSize',FntS); drawnow
    % Checking if limits have been reached
    if abs(lambda-lambdamax)<TOL*max(1,abs(lambda))|| ...
            abs(lambda-lambdamin)<TOL*max(1,abs(lambda))
        % Adjusting output arrays before leaving
        Lambdas=Lambdas(1:n+1); Ues=Ues(:,1:n+1); nUes=nUes(1:n+1);
        Tes=Tes(:,1:n+1); Iters=Iters(1:n+1);
        disp('LAMBDA LIMITS REACHED. STOPPING COMPUTATIONS')
        break
    end
    
    
end
disp(' ... done.')
disp('Do not forget to store your results somewhere.')