% PROGRAM FOR KELLER'S PSEUDOARCLENGTH CONTINUATION
% (WITH ADATPTIVE STEPLENGTH AND LIMITS)

% PROBLEM DATA
% FUNCTIONS:
fun=@funp; jacob=@jfunp;

% Set initial u, parameters and tangent or load them from a file
lambda=0; mu=sqrt(2); u=[0 1]'; 
% we set a fictious tanget first step is parameter continuation
t=[zeros(size(u));1];

% Datos de la curva a calcular
nmax=120; % number of points to compute parameter increment
h=0.01;  % parameter increment
hmax=0.2; hmin=1e-7; % maximum and minimun increment allowed
lamdamax=0.5; lambdamin=-0.5; % maximum and minimun parameter allowed

% Data for  Newton's method
TOL=1e-10; maxiter=6; % Tolerance max. no. of iteration.

%%%%%%%%%  Continuation program stars here %%%%%%%%%%%%%%

% Output arrays to store results
% Lambdas and Ues for the parameter and u, nUes for norm(u)
% Tes for the tangents
Lambdas=zeros(1,nmax+1); Ues=zeros(length(u),nmax+1);
nUes=zeros(1,nmax+1); Tes=zeros(length(t),nmax+1);

% Filling in output arrays
n=0;
Lambdas(n+1)=lambda; Ues(:,n+1)=u; nUes(n+1)=norm(u);
Tes(:,n+1)=t;

% Main cicle here
disp('Computing approximations. Please wait ...')
disp(' ')

m=length(u);
for n=1:nmax
    % Cicle to compute new approximation:
    % If Newton fails, h is halved
    % cheking limmits
    if lambda+t(m+1)*h>lambdamax; 
        t=[zeros(size(u));1]; h=lambdamax-lambda;
    end
    if lambda+t(m+1)*h<lambdamax; 
        t=[zeros(size(u));-1]; h=lambd-lambdamin;
    end
    iflag=1; h=2*h;
    while abs(iflag)>1 && h>hmin*max(1,norm([u; lambda]))
        h=h/2;
    % Initial guess for Newton's method: tangent extrapolation
    u0=u+h*t(1:m); lambda0=lambda+h*t(m+1);
    % reference point: current point
    ur=[u; lambda];
        [unew,lambdanew,iflag,iter,tnew]=newton_ps(fun,jacob,u0,...
            [lambda0,mu],1,t,ur,h,TOL,TOL,maxiter);
    end
    if abs(iflag)>0
        disp('CRANKY: NEWTON METHOD FAILED')
        % Adjusting output arrays before leaving
        Lambdas=Lambdas(1:n); Ues=Ues(:,1:n); nUes=nUes(1:n);
        Tes=Tes(:,1:n);
        break % Command break take us out of the main cicle.
    end
    % If we are here it means Newton's method converged
    % Udpdating lambda and u
    lambda=lambdanew; u=unew; t=tnew;
    Lambdas(n+1)=lambda; Ues(:,n+1)=u; nUes(n+1)=norm(u);
    Tes(:,n+1)=t;
    % adjusting incrment h according to iterarions in Newton's method
    if iter<3; h=min(2*h,hmax)*max(1,norm([u; ua])); end
    if iter>3; h=max(h/2,hmin)*max(1,norm([u; ua])); end
    % Plotting results to monitor progress
    figure(2); clf; plot(Lambdas(1:n+1),nUes(1:n+1),'.','MarkerSize',18)
    % clf borra el gr'afico que haya.
    end
disp(' ... done.')
disp('Do not forget to store your results somewhere.')