function [J,fp]=jfun_Euler(u,p)
% JFUN_EULER computes the Jacobian matrix
%            of function fun_Euler.
%  INPUT
%    u array of length N+1 with the angles
%        in U(1:N) and the Lagrange multiplier
%        in U(N+1)
%    p dimensionless force
%
%  OUTPUT
%    J (N+1)x(N+1) array with the Jacobian matrix
%        of function fun_Euler.
%    fp array the same size as u with fhe partial
%       derivative of fun_Euler wrt p.

N=length(u)-1; lambda=u(N+1);
% lambda is the  Lagrange multipliear of the restriction
% 
%    sum from n=1 up to n=N sin(u_n)/N = 0
%
sines=sin(u(1:N)); cosines=cos(u(1:N));

e=ones(N,1); D=spdiags(N*[-e 2*e -e],-1:1,N,N);
D(1,1)=-D(1,2); D(N,N)=-D(N,N-1); 
% Jacobian matrix is stored as a sparse matrix
% since it is a tridiagonal matrix.
D=D-(1/N)*spdiags(p*cosines+lambda*sines,0,N,N);
J=[D,cosines/N;(cosines')/N,0];

fp=[-sines/N; 0];
