function [f,Vx,Vy,Gx,Gy]=jrhsm(x,u,mu,Vx,Vy,Gx,Gy)
% LIKE JRHS BUT ALLOWS X TO BE A MATRIX
% JRHS COMPUTES THE ACTION OF THE JACOBIAN OF RHS
% JRHS COMPUTES THE ACTION OF THE JACOBIAN OF RHS
% returns in f J*x, where J is the jacobian of rhs
% if x and u are of the same length, or
% J*x(1:length(u),:) + fmu*x(length(u)+1,:) otherwise.
N=sqrt(length(u)+1);
U=reshape(u,N+1,N-1);
[m,n]=size(x);
if m>length(u); xkeep=x; xi=x(length(u)+1,:); x=x(1:length(u),:); end

X=reshape(x,N+1,N-1,n);
% The velocity of x in Xv Yv
% Inner product of 2x times e2 and velocity function basis in V;;
   J=[0:N]'*ones(1,N-1); K=ones(N+1,1)*[1:N-1];
   JK=sqrt(J.^2+K.^2); e=ones(1,n);
   JJ=kron(e,J); KK=kron(e,K); JJKK=sqrt(JJ.^2+KK.^2);
   JJ=reshape(JJ,N+1,N-1,n);KK=reshape(KK,N+1,N-1,n);
   JJKK=reshape(JJKK,N+1,N-1,n);
   Z=JJ.*X./JJKK; 
   % Nodal values of the velocity (/sqrt(mu))
% x component
   Xv=zeros(3*N/2-1,3*N/2+1,n);
   Xv(1:N,2:N,:)=-KK(2:N+1,:,:).*Z(2:N+1,:,:)./JJKK(2:N+1,:,:);
   Xv(N,:,:)=0.5*Xv(N,:,:);
   Xv=permute(ifct(permute(ifst(Xv),[2 1 3])),[2,1,3]);
% y component
   Yv=zeros(3*N/2+1,3*N/2-1,n);
   Yv(1:N+1,1:N-1,:)=JJ.*Z./JJKK; Yvkeep=Yv(1:N+1,1:N-1,:);
   Yv(N+1,:,:)=0.5*Yv(N+1,:,:);
   Yv=permute(ifst(permute(ifct(Yv),[2 1 3])),[2 1 3]);
% The gradient of x in Xg, Yg
% Partial wrt x;
   Xg=zeros(3*N/2-1,3*N/2-1,n);
   Xg(1:N,1:N-1,:)=-JJ(2:N+1,:,:).*X(2:N+1,:,:);
   Xg(N,:,:)=0.5*Xg(N,:,:);
% Partial wrt y;
   Yg=zeros(3*N/2+1,3*N/2+1,n);
   Yg(1:N+1,2:N,:)=KK.*X; Yg(N+1,:,:)=0.5*Yg(N+1,:,:);
% Nodal values of the gradient of the velocity
   Xg=permute(ifst(permute(ifst(pi*Xg),[2 1 3])),[2 1 3]); 
   %Yg=ifct(ifct(pi*Yg)')';
   Yg=permute(ifct(permute(ifct(pi*Yg),[2 1 3])),[2 1 3]); 


if nargin<5
% Inner product of 2u times e2 and velocity function basis in V
   J=[0:N]'*ones(1,N-1); K=ones(N+1,1)*[1:N-1];
   JK=sqrt(J.^2+K.^2);
   V=J.*U./JK;
   % Nodal values of the velocity (/sqrt(mu))
% x component
   Vx=zeros(3*N/2-1,3*N/2+1);
   Vx(1:N,2:N)=-K(2:N+1,:).*V(2:N+1,:)./JK(2:N+1,:);
   Vx(N,:)=0.5*Vx(N,:);
   Vx=ifct(ifst(Vx)')';
% y component
   Vy=zeros(3*N/2+1,3*N/2-1);
   Vy(1:N+1,1:N-1)=J.*V./JK; Vykeep=Vy(1:N+1,1:N-1);
   Vy(N+1,:)=0.5*Vy(N+1,:);
   Vy=ifst(ifct(Vy)')';
end
if nargin<7
% The gradient of the Temperature in Gx, Gy
% Partial wrt x;
   Gx=zeros(3*N/2-1,3*N/2-1);
   Gx(1:N,1:N-1)=-J(2:N+1,:).*U(2:N+1,:);
   Gx(N,:)=0.5*Gx(N,:);
% Partial wrt y;
   Gy=zeros(3*N/2+1,3*N/2+1);
   Gy(1:N+1,2:N)=K.*U; Gy(N+1,:)=0.5*Gy(N+1,:);
% Nodal values of the gradient of the velocity
   Gx=ifst(ifst(pi*Gx)')'; Gy=ifct(ifct(pi*Gy)')';
end
VVx=kron(e,Vx); VVy=kron(e,Vy); GGx=kron(e,Gx); GGy=kron(e,Gy);
VVx=reshape(VVx,3*N/2-1,3*N/2+1,n); VVy=reshape(VVy,3*N/2+1,3*N/2-1,n); 
GGx=reshape(GGx,3*N/2-1,3*N/2-1,n); GGy=reshape(GGy,3*N/2+1,3*N/2+1,n);
% The nonlinear term
   R=zeros(3*N/2+1,3*N/2-1,n);
   R(2:3*N/2,:,:)=VVx(:,2:3*N/2,:).*Xg;
   R=R+VVy.*Yg(:,2:3*N/2,:);
   R(2:3*N/2,:,:)=R(2:3*N/2,:,:)+Xv(:,2:3*N/2,:).*GGx;
   R=R+Yv.*GGy(:,2:3*N/2,:);
   R=permute(fst(permute(fct(R),[2 1 3])),[2 1 3]);
   R=R(1:N+1,1:N-1,:); R(N+1,:,:)=2*R(N+1,:,:);
   %fmu=reshape(R-(0.5/sqrt(mu))*Vykeep,N^2-1,1);
% Minus the Laplacian
   Lap=(pi^2)*(JJ.^2+KK.^2).*X;
% The rhs
   F=Lap+mu*(R-Yvkeep);
   f=reshape(F,N^2-1,n);
   
% the partial derivative of rhs wrt mu
if exist('xi')
    R2=zeros(3*N/2+1,3*N/2-1);
    R2(2:3*N/2,:)=Vx(:,2:3*N/2).*Gx;
    R2=R2+Vy.*(Gy(:,2:3*N/2)-1);
    R2=fst(fct(R2)')'; R2=R2(1:N+1,1:N-1); R2(N+1,:)=2*R2(N+1,:);
    fmu=reshape(R2,N^2-1,1);
    f=f+fmu*xi;
end

   