function [f,Vx,Vy,Gx,Gy]=jrhs(x,u,mu,Vx,Vy,Gx,Gy)
% returns in f J*x, where J is the jacobian of rhs
N=sqrt(length(u)+1);
U=reshape(u,N+1,N-1);
X=reshape(x,N+1,N-1);
% The velocity of x in Xv Yv
% Inner product of 2x times e2 and velocity function basis in V;;
   J=[0:N]'*ones(1,N-1); K=ones(N+1,1)*[1:N-1];
   JK=sqrt(J.^2+K.^2);
   Z=J.*X./JK;
   % Nodal values of the velocity (/sqrt(mu))
% x component
   Xv=zeros(3*N/2-1,3*N/2+1);
   Xv(1:N,2:N)=-K(2:N+1,:).*Z(2:N+1,:)./JK(2:N+1,:);
   Xv(N,:)=0.5*Xv(N,:);
   Xv=ifct(ifst(Xv)')';
% y component
   Yv=zeros(3*N/2+1,3*N/2-1);
   Yv(1:N+1,1:N-1)=J.*Z./JK; Yvkeep=Yv(1:N+1,1:N-1);
   Yv(N+1,:)=0.5*Yv(N+1,:);
   Yv=ifst(ifct(Yv)')';
% The gradient of x in Xg, Yg
% Partial wrt x;
   Xg=zeros(3*N/2-1,3*N/2-1);
   Xg(1:N,1:N-1)=-J(2:N+1,:).*X(2:N+1,:);
   Xg(N,:)=0.5*Xg(N,:);
% Partial wrt y;
   Yg=zeros(3*N/2+1,3*N/2+1);
   Yg(1:N+1,2:N)=K.*X; Yg(N+1,:)=0.5*Yg(N+1,:);
% Nodal values of the gradient of the velocity
   Xg=ifst(ifst(pi*Xg)')'; Yg=ifct(ifct(pi*Yg)')';


if nargin<5
% Inner product of 2u times e2 and velocity function basis in V
   J=[0:N]'*ones(1,N-1); K=ones(N+1,1)*[1:N-1];
   JK=sqrt(J.^2+K.^2);
   V=J.*U./JK;
   % Nodal values of the velocity (/sqrt(mu))
% x component
   Vx=zeros(3*N/2-1,3*N/2+1);
   Vx(1:N,2:N)=-K(2:N+1,:).*V(2:N+1,:)./JK(2:N+1,:);
   Vx(N,:)=0.5*Vx(N,:);
   Vx=ifct(ifst(Vx)')';
% y component
   Vy=zeros(3*N/2+1,3*N/2-1);
   Vy(1:N+1,1:N-1)=J.*V./JK; Vykeep=Vy(1:N+1,1:N-1);
   Vy(N+1,:)=0.5*Vy(N+1,:);
   Vy=ifst(ifct(Vy)')';
end
if nargin<7
% The gradient of the Temperature in Gx, Gy
% Partial wrt x;
   Gx=zeros(3*N/2-1,3*N/2-1);
   Gx(1:N,1:N-1)=-J(2:N+1,:).*U(2:N+1,:);
   Gx(N,:)=0.5*Gx(N,:);
% Partial wrt y;
   Gy=zeros(3*N/2+1,3*N/2+1);
   Gy(1:N+1,2:N)=K.*U; Gy(N+1,:)=0.5*Gy(N+1,:);
% Nodal values of the gradient of the velocity
   Gx=ifst(ifst(pi*Gx)')'; Gy=ifct(ifct(pi*Gy)')';
end
% The nonlinear term
   R=zeros(3*N/2+1,3*N/2-1);
   R(2:3*N/2,:)=Vx(:,2:3*N/2).*Xg;
   R=R+Vy.*Yg(:,2:3*N/2);
   R(2:3*N/2,:)=R(2:3*N/2,:)+Xv(:,2:3*N/2).*Gx;
   R=R+Yv.*Gy(:,2:3*N/2);
   R=fst(fct(R)')'; R=R(1:N+1,1:N-1); R(N+1,:)=2*R(N+1,:);
   %fmu=reshape(R-(0.5/sqrt(mu))*Vykeep,N^2-1,1);
% Minus the Laplacian
   Lap=(pi^2)*(J.^2+K.^2).*X;
% The rhs
   F=Lap+mu*(R-Yvkeep);
   f=reshape(F,N^2-1,1);
   

   