function [f,fmu,Vx,Vy,Gx,Gy]=rhsa(u,mu)
% RHS OF CONVECTION PROBLEM IN POROUS MEDIA
% in Vx Vy the nodal values of the velocity (/sqrt(mu))
% in Gx Gy the nodal values of the partial derivatives of u
global MU VX VY GX GY
N=sqrt(length(u)+1);
U=reshape(u,N+1,N-1);
% Inner product of 2u times e2 and velocity function basis in V;
   J=[0:N]'*ones(1,N-1); K=ones(N+1,1)*[1:N-1];
   JK=sqrt(J.^2+K.^2);
   V=J.*U./JK;
% The gradient of the Temperature in Gx, Gy
% Partial wrt x;
   Gx=zeros(3*N/2-1,3*N/2-1);
   Gx(1:N,1:N-1)=-J(2:N+1,:).*U(2:N+1,:);
   Gx(N,:)=0.5*Gx(N,:);
% Partial wrt y;
   Gy=zeros(3*N/2+1,3*N/2+1);
   Gy(1:N+1,2:N)=K.*U; Gy(N+1,:)=0.5*Gy(N+1,:);
% Nodal values of the gradient of the velocity
   Gx=ifst(ifst(pi*Gx)')'; Gy=ifct(ifct(pi*Gy)')';
% Nodal values of the velocity (/sqrt(mu))
% x component
   Vx=zeros(3*N/2-1,3*N/2+1);
   Vx(1:N,2:N)=-K(2:N+1,:).*V(2:N+1,:)./JK(2:N+1,:);
   Vx(N,:)=0.5*Vx(N,:);
   Vx=ifct(ifst(Vx)')';
% y component
   Vy=zeros(3*N/2+1,3*N/2-1);
   Vy(1:N+1,1:N-1)=J.*V./JK; Vykeep=Vy(1:N+1,1:N-1);
   Vy(N+1,:)=0.5*Vy(N+1,:);
   Vy=ifst(ifct(Vy)')';
% The nonlinear term
   R=zeros(3*N/2+1,3*N/2-1);
   R(2:3*N/2,:)=Vx(:,2:3*N/2).*Gx;
   R=R+Vy.*Gy(:,2:3*N/2);
   R=fst(fct(R)')'; R=R(1:N+1,1:N-1); R(N+1,:)=2*R(N+1,:);
   fmu=reshape(R-Vykeep,N^2-1,1);
% Minus the Laplacian
   Lap=(pi^2)*(J.^2+K.^2).*U;
% The rhs
   F=Lap+mu*(R-Vykeep);
   f=reshape(F,N^2-1,1);
% updatying global variables
MU=mu; GX=Gx; GY=Gy; VX=Vx; VY=Vy;

   